//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "MailboxWins.h"
#include "PickBoxDlgs.h"
#include "NewNameDlgs.h"
#include "MsgWins.h"
#include "MsgInWin.h"
#include "FileMan.h"
#include "Global.h"
#include "HeadMan.h"
#include "pop3mail.h"
//---------------------------------------------------------------------------
#pragma link "Grids"
#pragma resource "*.dfm"
TMailboxWin *MailboxWin;
//---------------------------------------------------------------------------
TMailboxWin* __fastcall OpenMailbox(AnsiString &Boxname)
{
   if (Boxname.IsEmpty()) return NULL;
   if ((MailboxWin=FindOpenMailbox(Boxname))==NULL)
      MailboxWin = new TMailboxWin(Application->MainForm,Boxname);
   else
      MailboxWin->BringToFront();
   return MailboxWin;
}
//---------------------------------------------------------------------------
TMailboxWin* __fastcall FindOpenMailbox(AnsiString &Boxname)
{
   TMailboxWin *mw=NULL;
   TForm *mf=Application->MainForm;
   for (int i=0; i<mf->MDIChildCount; i++)
      if ((mw=dynamic_cast<TMailboxWin*>(mf->MDIChildren[i]))!=NULL)
      {
         if (mw->Boxname()==Boxname) break;
         mw = NULL;
      }
   return mw;
}
//---------------------------------------------------------------------------
AnsiString __fastcall CreateMailbox(AnsiString &Boxname)
{
   AnsiString boxname=Boxname;
   if (boxname.IsEmpty()) boxname = GetMailboxName();
   if (boxname.IsEmpty()) return "";

   TStringList *emptybox = new TStringList();
   PutAppData(UserMailboxes+PS+boxname,emptybox);
   SetMailboxPos(boxname,0);
   AddMenubox(boxname);
   delete emptybox;
   return boxname;
}
//---------------------------------------------------------------------------
void __fastcall DelMailbox(AnsiString &Boxname)
{
   // remove all messages
   TStringList *mail = new TStringList();
   DelMessages(Boxname,NULL);
   // remove empty mailbox
   PutAppData(UserMailboxes+PS+Boxname,NULL);
   DelMailboxPos(Boxname);
   DelMenubox(Boxname);
   delete mail;
}
//---------------------------------------------------------------------------
bool __fastcall MailboxExists(AnsiString &Boxname)
{
   TStringList *list = new TStringList();
   GetAppData(list,UserMailboxes);
   bool exist = list->IndexOf(Boxname)>=0;
   delete list;
   return exist;
}
//---------------------------------------------------------------------------
bool __fastcall MenuboxExists(AnsiString &Boxname)
{
   TStringList *list = new TStringList();
   GetAppData(list,MailboxBoxmenu);
   bool exist = list->IndexOf(Boxname)>=0;
   delete list;
   return exist;
}
//---------------------------------------------------------------------------
void __fastcall SetMailboxPos(AnsiString &Boxname, long position)
{
   TStringList *pos = new TStringList();
   GetAppData(pos,MailboxPos);
   pos->Values[Boxname] = s((int)position);
   PutAppData(MailboxPos,pos);
   delete pos;
}
//---------------------------------------------------------------------------
void __fastcall DelMailboxPos(AnsiString &Boxname) // deletes mailbox/position
{
   TStringList *all = new TStringList();
   GetAppData(all,UserMailboxes);
   TStringList *pos = new TStringList();
   GetAppData(pos,MailboxPos);
   int p=pos->IndexOf(Boxname);
   if (p>=0) pos->Delete(p);
// look for lingering box poses--remove at some time in the future
   p=0; while (p<pos->Count)
   {
      if (all->IndexOf(pos->Strings[p])>=0) pos->Delete(p);
      else p++;
   }
   PutAppData(MailboxPos,pos);
   delete all;
   delete pos;
}
//---------------------------------------------------------------------------
void __fastcall AddMenubox(AnsiString &Boxname)
{
   TStrings *menu = new TStringList();
   GetAppData(menu,MailboxBoxmenu);
   if (menu->IndexOf(Boxname)<0)
   {
      menu->Add(Boxname);
      PutAppData(MailboxBoxmenu,menu);
   }
   Application->MainForm->Perform(WM_LOADBOXMENU,0,NULL);
   delete menu;
}
//---------------------------------------------------------------------------
bool __fastcall DelMenubox(AnsiString &Boxname)
{
   int pos;
   bool del=false;
   TStrings *menu = new TStringList();
   GetAppData(menu,MailboxBoxmenu);
   if ((pos=menu->IndexOf(Boxname))>=0)
   {
      menu->Delete(pos);
      PutAppData(MailboxBoxmenu,menu);
      Application->MainForm->Perform(WM_LOADBOXMENU,0,NULL);
      del = true;
   }
   delete menu;
   return del;
}
//---------------------------------------------------------------------------
// TMailGrid == Mailbox Index Window
//---------------------------------------------------------------------------
__fastcall TMailGrid::TMailGrid(TComponent *Owner)
   : TDrawGrid(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMailGrid::TMailGrid(TComponent *Owner, AnsiString &BoxName)
   : TDrawGrid(Owner)
{
   Mailbox = BoxName; Parent = dynamic_cast<TWinControl*>(Owner);
   Align = alClient;
   DefaultDrawing = false;
   DefaultRowHeight = 18;
   FixedRows = 1;
   Options << goRowSelect << goThumbTracking;
   Options >> goRangeSelect;
   ScrollBars = ssBoth;
   RowHeights[0] = 22;
   ColCount = HasPop3() ? 4 : 3;
   FixedCols = 0;
   if (HasPop3())
   {
      ColWidths[0] = 20;
      ColWidths[1] = 108;//117;
      ColWidths[2] = 180;
      ColWidths[3] = Width;
   }
   else
   {
      ColWidths[0] = 108;
      ColWidths[1] = 180;
      ColWidths[2] = Width;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::SelectAll()
{
   for (int i=1; i<RowCount; i++) Selected[i] = true;
   for (int i=TopRow; i<=TopRow+VisibleRowCount; i++)
      InvalidateRow(i);
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::DrawCell(long ACol, long ARow, const Windows::TRect &Rect,
   TGridDrawState State)
{
   if (ARow<0 || ARow>RowCount-1) return;

   TColor prev;
   RECT r; SetRect(&r,Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
   AnsiString text,str=MailList->Strings[ARow];
   AnsiString DataKey = UserMailboxes+PS+Mailbox+PS+str;
   if (HasPop3())
   {
      switch (ACol)
      {
         case 0: text = GetMsgStatus(DataKey); break;
         case 1: text = MessageDate(str); break;
         case 2: text = MessageWho(str); break;
         case 3: text = MessageSubject(str); break;
      }
   }
   else
   {
      switch (ACol)
      {
         case 0: text = MessageDate(str); break;
         case 1: text = MessageWho(str); break;
         case 2: text = MessageSubject(str); break;
      }
   }
   Canvas->Font->Color = clWindowText;
   if (ARow==0)
   {
      RECT h; SetRect(&h,Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
      TPoint pts[6] = {
         {h.left,h.bottom-1},{h.left,h.top},{h.right,h.top},
         {h.right-1,h.top+1},{h.right-1,h.bottom-1},{h.left,h.bottom-1}
      };
      Canvas->Pen->Color = clBtnHighlight;
      Canvas->Polyline(&pts[0],2);
      Canvas->Pen->Color = clBtnShadow;
      Canvas->Polyline(&pts[3],2);
      Canvas->Brush->Color = clBtnFace;
      InflateRect(&h,-1,-1);
      Canvas->FillRect(TRect(h));
      r.top++;
   }
   else
   if (Selected[ARow])
   {
      Canvas->Font->Color = clHighlightText;
      Canvas->Brush->Color = clHighlight;
      Canvas->FillRect(TRect(r));
   }
   else
   {
      Canvas->Font->Color = clWindowText;
      Canvas->Brush->Color = clWindow;
      Canvas->FillRect(TRect(r));
   }

   InflateRect(&r,-1,-2); r.right-=3;
   if (HasPop3() && ACol==0 && ARow>0 && text=="N")
   {
      prev = Canvas->Brush->Color;
      Canvas->Brush->Color = clRed;
      Canvas->Ellipse(r.left+5,r.top+3,r.left+13,r.top+11);
      Canvas->Brush->Color = prev;
   }
   else
   {
      Canvas->TextRect(
         TRect(r),
         r.left+5,
         r.top,
         text
       );
   }

   if (ARow==Row && ARow!=0)
   {
      TRect fr=Rect; fr.Right++;
      Canvas->DrawFocusRect(fr);
   }
}
//---------------------------------------------------------------------------
__fastcall TMailGrid::~TMailGrid()
{
   CloseIndex();
}
//---------------------------------------------------------------------------
#define IDXSTRING(p,l) IndexStr.SubString(p,l)
AnsiString __fastcall TMailGrid::MessageDate(AnsiString &IndexStr)
{
   if (IndexStr==HeaderText) return GetParam(0,IndexStr," ");
   if (IndexStr.IsEmpty()) return "";

   AnsiString h,ampm="am";
   int H = IDXSTRING(12,2).ToIntDef(0);
   if (H>12) { H -= 12; ampm = "pm"; }
   h = H; if (h.Length()<2) h = "0"+h;
   return IDXSTRING(3,8)+"  "+h+":"+IDXSTRING(15,2)+" "+ampm+"  "+IndexStr.SubString(21,2);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMailGrid::MessageWho(AnsiString &IndexStr)
{
   if (IndexStr==HeaderText) return GetParam(1,IndexStr," "); // header
   if (IndexStr.IsEmpty()) return "";

   AnsiString who,ic=InCode,oc=OutCode;
   if (IndexStr[23]==ic[1] || IndexStr[23]==oc[1])
      who=HeadMan->GetAddress(GetParam(1,IndexStr,FieldSep.c_str()));
   else
      who=GetParam(3,IndexStr," "); // original style index

   return who;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMailGrid::MessageSubject(AnsiString &IndexStr)
{
   if (IndexStr==HeaderText) return GetParam(2,IndexStr," "); // header
   if (IndexStr.IsEmpty()) return "";

   int len,pos,fld;
   AnsiString sub,ic=InCode,oc=OutCode;
   if (IndexStr[23]==ic[1] || IndexStr[23]==oc[1]) // new style
      sub=GetParam(2,IndexStr,FieldSep.c_str());
   else
   {
      // original style
      len=IndexStr.Length();
      pos=len+1;
      fld=-2;
      for (int i=0; i<len; i++)
      {
         if (IndexStr[i]==' ') fld++;
         if (fld==(hPopLib?3:2))
         {
            pos = i+1;
            break;
         }
      }
      sub=IndexStr.SubString(pos,256);
   }
   return sub;
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::OpenIndex(bool sorted)
{
   MailList = new TStringList(); MailList->Sorted = sorted;
   GetAppData(MailList,UserMailboxes+PS+BoxName());
   MailList->Sorted = false;
   if (MailList->Count)
      MailList->Insert(0,HeaderText);
   else
      MailList->Append(HeaderText);

   RowCount = MailList->Count;
   Selected = new bool[RowCount];
   ClearSelected();

   long top=TopRow, row=GridRow(GetBoxPos());
   if (row!=0 && top!=0)
   {
      top = RowCount<=VisibleRowCount ? 1 : row-VisibleRowCount/2;
      if (top<1) top = 1;
   }
   Selected[row] = true;
   Anchor = row;
   TopRow = top;
   Row = row;
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::CloseIndex()
{
   SetBoxPos(Row);
   delete [] Selected;
   delete MailList;
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::RefreshIndex()
{
   long toprow = TopRow;
   CloseIndex(); OpenIndex();
   if (toprow>=0 && toprow<RowCount) TopRow = toprow;
   if (RowCount && RowCount<=VisibleRowCount) TopRow = 0;
   for (int i=TopRow; i<=TopRow+VisibleRowCount; i++)
      InvalidateRow(i);
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::SetBoxPos(int pos)
{
   TStringList *poses = new TStringList();
   GetAppData(poses,MailboxPos);
   poses->Values[BoxName()] = s(pos);
   PutAppData(MailboxPos,poses);
   delete poses;
}
//---------------------------------------------------------------------------
int __fastcall TMailGrid::GetBoxPos()
{
   TStringList *poses = new TStringList();
   GetAppData(poses,MailboxPos);
   int pos = poses->Values[BoxName()].ToIntDef(-1);
   delete poses;
   return pos;
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::SelCount()
{
   long sels=0;
   for (long i=1;i<RowCount; i++)
      if (Selected[i]) sels++;
   return sels;
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::TransferSelected(AnsiString &NewBoxname)
{
   long count=TransferMessages(Mailbox,Selected,NewBoxname);
   RefreshIndex();
   return count;
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::DeleteSelected()
{
   long count=DelMessages(Mailbox,Selected);
   RefreshIndex();
   return count;
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::ClearSelected()
{
   long sels=0;
   for (long i=1; i<RowCount; i++)
      if (Selected[i])
      {
          Selected[i] = false;
          sels++;
      }
   return sels;
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::OpenSelected()
{
   long sels=0;
   for (long i=1; i<RowCount; i++)
      if (Selected[i])
      {
         if (IsIncoming(UserMailboxes+PS+Mailbox+PS+MailList->Strings[i]))
            OpenMessageIn(Mailbox,MailList->Strings[i]);
         else
            OpenMessage(Mailbox,MailList->Strings[i]);
         sels++;
      }
   return sels;
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::MouseDown(Controls::TMouseButton Button,
   Classes::TShiftState Shift, int X, int Y)
{
   if (Button==mbRight || Button==mbMiddle) return;
   if (Button==mbLeft && Shift.Contains(ssDouble) && !Shift.Contains(ssCtrl))
   {
      DblClick();
      return;
   }

   long col,row;
   MouseToCell(X,Y,col,row);
   if (row<1 || Row>RowCount-1) return;

   Row = row;
   if (Shift.Contains(ssShift))
   {
      SetSelected(Anchor,Row);
   }
   else
   if (Shift.Contains(ssCtrl))
   {
      Selected[Row] = !Selected[Row];
      InvalidateRow(Row);
      Anchor = Row;
   }
   else
   {
      SetSelected(Row);
      Anchor = Row;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::DblClick()
{
   OpenSelected();
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::KeyDown(unsigned short &Key, TShiftState Shift)
{
   if (Key==VK_SHIFT || Key==VK_CONTROL)// || Key==VK_MENU)
      return;
   if (Key==VK_SPACE)
   {
      Selected[Row] = !Selected[Row];
      Anchor = Row;
      InvalidateRow(Row);
      return;
   }
   if (Key==VK_RETURN)
   {
      DblClick();
      return;
   }

   long newrow=Row;
   switch (Key)
   {
      case VK_DOWN:
      case VK_RIGHT:
         newrow = GridRow(Row+1);
         break;
      case VK_UP:
      case VK_LEFT:
         newrow = GridRow(Row-1);
         break;
      case VK_NEXT:
         newrow = GridRow(Row+VisibleRowCount-1);
         break;
      case VK_PRIOR:
         newrow = GridRow(Row-VisibleRowCount-1);
         break;
      case VK_HOME:
         newrow = 1;
         break;
      case VK_END:
         newrow = RowCount-1;
         break;
   }
   if (newrow!=Row)
   {
      if (Shift.Contains(ssShift))
         SetSelected(Anchor,newrow);
      else
      {
         SetSelected(newrow);
         Anchor = newrow;
         Row = newrow;
      }
      Row = newrow;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMailGrid::SetSelected(long Anchor, long ToRow)
{
   bool sel,val;
   if (ToRow==-1)
   {
      for (long i=0; i<RowCount; i++)
      {
         sel = i==Anchor; val = Selected[i] != sel;
         Selected[i] = sel;
         if (val) InvalidateRow(i);
      }
   }
   else
   if (Anchor<ToRow)
   {
      for (long i=0; i<RowCount; i++)
      {
         sel = i>=Anchor && i<=ToRow; val = Selected[i]!=sel;
         if (i!=Anchor) Selected[i] = sel;
         if (val) InvalidateRow(i);
      }
   }
   else
   {
      for (long i=0; i<RowCount; i++)
      {
         sel = i>=ToRow && i<=Anchor; val = Selected[i] != sel;
         if (i!=Anchor) Selected[i] = sel;
         if (val) InvalidateRow(i);
      }
   }
}
//---------------------------------------------------------------------------
long __fastcall TMailGrid::GridRow(long row)
{
   if (row>RowCount-1) row = RowCount-1;
   if (row<0) row = 0;
   if (row==0 && RowCount>1) row=1;
   return row;
}
//---------------------------------------------------------------------------
// TMailboxWin == Mailbox Index Window
//---------------------------------------------------------------------------
__fastcall TMailboxWin::TMailboxWin(TComponent* Owner)
   : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMailboxWin::TMailboxWin(TComponent* Owner, AnsiString &Mailbox)
   : TForm(Owner)
{
   Caption = Mailbox;
   PopupMailboxMenuItem->Checked = MenuboxExists(Mailbox);
   MailGrid = new TMailGrid(this,Mailbox);
   MailGrid->PopupMenu = PopupMenu;
   MailGrid->OpenIndex();
   if (Mailbox=="outbox")
   {
      PopupMailboxMenuItem->Checked = true;
      PopupMailboxMenuItem->Enabled = false;
   }
   FormResize(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   delete MailGrid;
   MailGrid = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::FormClose(TObject *Sender, TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::RefreshIndex()
{
   MailGrid->RefreshIndex();
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupMenuPopup(TObject *Sender)
{
   long count=MailGrid->SelCount();
   PopupSortedItem->Enabled = count<2;
   PopupTransferItem->Enabled = count>0;
   PopupDeleteItem->Enabled = count>0;
   PopupOpenItem->Enabled = count>0;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupMailboxMenuItemClick(TObject *Sender)
{
   TMenuItem *item = PopupMailboxMenuItem;
   item->Checked = !item->Checked;
   if (item->Checked) AddMenubox(Caption); else DelMenubox(Caption);
}
//-------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupRefreshItemClick(TObject *Sender)
{
   MailGrid->RefreshIndex();
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::FormResize(TObject *Sender)
{
   if (WindowState == wsNormal || (WindowState==wsMaximized && MailGrid))
      MailGrid->ColWidths[hPopLib?3:2] = MailGrid->ClientWidth-1;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupOpenItemClick(TObject *Sender)
{
   MailGrid->OpenSelected();
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupDeleteItemClick(TObject *Sender)
{
   if (MsgBoxHwnd(Handle,"Delete selected message(s)!\n\nAre you sure?",
       "Delete Message",MB_OKCANCEL)==IDOK)
   {
      MailGrid->Cursor = crHourGlass;
      SetRedraw(MailGrid,false);
      MailGrid->DeleteSelected();
      SetRedraw(MailGrid,true);
      MailGrid->Cursor = crDefault;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupTransferItemClick(TObject *Sender)
{
   TMailboxWin *mw;
   AnsiString newbox;
   if ((newbox=GetTransferBoxname(Boxname()))!="")
   {
      MailGrid->Cursor = crHourGlass;
      SetRedraw(MailGrid,false);
      if (MailGrid->TransferSelected(newbox)>0)
      {
         MailGrid->RefreshIndex();
         if ((mw=FindOpenMailbox(newbox))!=NULL)
         {
            SetRedraw(mw->MailGrid,false);
            mw->MailGrid->RefreshIndex();
            SetRedraw(mw->MailGrid,true);
         }
      }
      SetRedraw(MailGrid,true);
      MailGrid->Cursor = crDefault;
   }
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupDeleteMailboxItemClick(TObject *Sender)
{
   AnsiString boxname = Caption;
   if (MsgBoxHwnd(Handle,"Delete mailbox '"+boxname+"' and contents!\n\nAre you sure?",
          "Delete Mailbox",MB_OKCANCEL)==IDOK &&
       MsgBoxHwnd(Handle,"Mail in '"+boxname+"' will be lost forever!",
          "Delete Mailbox",MB_OKCANCEL)==IDOK)
   {

      Cursor = crHourGlass;
      DelMailboxPos(boxname);
      DelMailbox(boxname);
      DelMenubox(boxname);
      Close();
   }
}
//---------------------------------------------------------------------------
long __fastcall TMailboxWin::GoNewMessages()
{
   if (MailGrid->RowCount==0) return -1;

   AnsiString MasterKey = UserMailboxes+PS+MailGrid->Mailbox;
   long row = MailGrid->RowCount-1; MailGrid->Row = row;
   while (row>1 && GetMsgStatus(MasterKey+PS+MailGrid->MailList->Strings[row-1])=='N')
      row--;
   MailGrid->Row = row;
   MailGrid->Selected[row] = true;
   return row;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::EditSelectAllItemClick(TObject *Sender)
{
   MailGrid->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopSelectAllItemClick(TObject *Sender)
{
   MailGrid->SelectAll();
}
//---------------------------------------------------------------------------
/*
void __fastcall TMailGrid::TopLeftChanged(TObject *Sender)
{
   if (LeftCol==LastLeftCol) return;
   bool locked = LockWindowUpdate(AppMainForm->Handle);

   int i,w=0,lastcol=hPopLib?3:2;
   for (i=LeftCol; i<lastcol; i++) w += ColWidths[i];
   ColWidths[lastcol] = ClientWidth-(w+(i-1));

   if (LeftCol==lastcol) ColWidths[lastcol+1] = 0;
   else ColWidths[lastcol+1]=1000;
   AppProcessMessages();
   if (locked) LockWindowUpdate(0);
//   sm((int)LeftCol);
   LastLeftCol = LeftCol;
}*/
//---------------------------------------------------------------------------
void __fastcall TMailGrid::SortList(bool sorted)
{
   AnsiString sel,cap;
   TForm *f = dynamic_cast<TForm*>(Owner);
   cap = f->Caption;
   f->Caption = sorted ? "Sorting...":"Unsorting...";

   SetRedraw(this,false);
   for (int i=1; i<MailList->Count; i++)
      if (Selected[i])
      {
         sel = MailList->Strings[i];
         break;
      }
   ClearSelected();
   MailList->Clear();

   MailList->Sorted = sorted;
   GetAppData(MailList,UserMailboxes+PS+BoxName());
   MailList->Sorted = false;

   if  (MailList->Count) MailList->Insert(0,HeaderText);
   else MailList->Append(HeaderText);
   RowCount = MailList->Count;

   Row=1;
   TopRow=0; // get visible rows with toprow at 0--tells true if less than maxvis
   int visible=VisibleRowCount;
   for (int i=0; i<MailList->Count; i++)
      if (MailList->Strings[i]==sel) // used this technique--IndexOf and Find????
      {
         Row = i;
         break;
      }

   Selected[Row] = true;
   TopRow = Row - (visible/2);
   if (TopRow<1 || RowCount<=visible) TopRow = 1;
   SetRedraw(this, true);
   Anchor = Row;
   Sleep(200);
   f->Caption = cap;
}
//---------------------------------------------------------------------------
void __fastcall TMailboxWin::PopupSortedItemClick(TObject *Sender)
{
   AnsiString sel;
   bool checked = !PopupSortedItem->Checked;
   PopupSortedItem->Checked = checked;
   MailGrid->SortList(checked);
}
//---------------------------------------------------------------------------
